<?php

namespace App\Wrangler\Components;

use Illuminate\Http\Request;
use Medialight\Textstem\Models\Post;
use Medialight\Textstem\Models\WranglerPage;
use Medialight\Textstem\Wrangler\ActiveComponent;

class News extends ActiveComponent
{
    public $config;

    //    public static string $description="Displays a list of news items, or a single view if a slug can be identified in the url.";

    public array $editSetup = [
        [
            'controltype' => 'instructions',
            'value' => 'Displays a list of news items, or a single view if a slug can be identified in the url',
        ],
        [
            'name' => 'title',
            'controltype' => 'text',
            'value' => '',
            'label' => 'Title',
            'instructions' => 'Optional title. Note - this text will be escaped. To enter raw HTML, use the HTML block.',
        ],

        [
            'name' => 'listtemplate',
            'controltype' => 'select',
            'value' => 'default',
            'label' => 'List template',
            'options' => ['list',  'cards'],
            'size' => 2,
        ],
        [
            'name' => 'viewtemplate',
            'controltype' => 'select',
            'value' => 'default',
            'label' => 'View template',
            'options' => ['single'],
            'size' => 2,
        ],
        [
            'name' => 'viewurl',
            'controltype' => 'text',
            'value' => '',
            'label' => 'View Page Url',
            'instructions' => 'Optional - if left empty, he current page is used to display the single view.',
        ],
        [
            'name' => 'limit',
            'controltype' => 'select',
            'value' => '5',
            'label' => 'Page Size',
            'instructions' => 'Number of posts to show per page (default 5)',
            'casts' => 'int',
            'options' => [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 12, 20, 50],
            'size' => 1,
        ],
//        [
//            'name' => 'show_pagination',
//            'controltype' => 'yesno',
//            'value' => '0',
//            'label' => 'Show Pager',
//            'size' => 1,
//        ],
    ];

    //    public function __construct($config = [])
    //    {
    //
    //        $this->checkEditorSetup($this->editSetup);
    //
    //        $this->config = $this->setupConfig($config);
    //
    //
    //        foreach ($this->editSetup as $indx => &$setup) {
    //            $field = $setup['name'] ?? 'conf_' . $indx;
    //            if (isset($this->config[$field])) {
    //                // set the value
    //                $setup['value'] = $this->config[$field];
    //            } else {
    //                // look in the data
    //                if (is_array($this->config['data']) && isset($this->config['data'][$field])) {
    //                    $setup['value'] = $this->config['data'][$field];
    //
    //                } else {
    //                   //   \Barryvdh\Debugbar\Facades\Debugbar::addMessage('no value for ' . $field . ' in ', $this->config);
    //                }
    //            }
    //        }
    //
    //    }

    public function render(Request $request, WranglerPage $wranglerPage)
    {
        $slug =   \Medialight\Textstem\Wrangler\Request::get('view');
        if ($slug) {
            // trying to look at single view
            $post = Post::where('slug', $slug)->first();
            if ($post) {
                $data = [
                    'config' => $this->config,
                    'post' => $post,
                    'title' => $post->title,
                    'route_index' => $wranglerPage->url,
                ];
                // now tweak the page
                $pageProps = collect($post);

                // the assumeFeatures method allows the page to take on the features of the post
                $wranglerPage->assumeFeatures($post);

                $view = $this->singleListView();

                return view("$view", $data)->render();
            } else {
                abort(404);
            }

        }

        $posts = Post::public()
            ->orderBy('published_at', 'desc')
            ->paginate($this->config['data']['limit'] ?? 5)
            ->withQueryString();
        $data = [
            'config' => $this->config,
            'posts' => $posts,
            'title' => 'Latest News',
            'route' => $wranglerPage->url.'/'.($this->config['slug'] ?? $this->config['id']),
        ];
        $view = $this->getListView();
        if(!view()->exists($view)){
            $view = 'wrangler.components.news.cards';
        }

        return view($view, $data)->render();
    }

    public function preview()
    {
        $posts = Post::latest()
            ->paginate($this->config['data']['limit'] ?? 5)
            ->withQueryString();

        $data = [
            'config' => $this->config,
            'posts' => $posts,
            'title' => 'Latest News',
            'route' => '#',
        ];
        $view = $this->getListView();
        if(!view()->exists($view)){
            $view = 'wrangler.components.news.list';
        }
        return view($view, $data)->render();
    }

    private function singleListView()
    {
        $view = 'wrangler.components.news.'.strtolower($this->config['data']['viewtemplate'] ?? 'single');

        // check it exists?
        return $view;
    }

    private function getListView()
    {
        $view = 'wrangler.components.news.'.strtolower($this->config['data']['listtemplate'] ?? 'list');
        if(!view()->exists($view)){
            $view = 'wrangler.components.news.list';
        }
        // check it exists?
        return $view;
    }
}
